'''
read about twitter entities here
https://dev.twitter.com/overview/api/entities-in-twitter-objects
'''

class TweetParser(object):
	def __init__(self):
		pass
	'''
    	returns True if tweet contains one or more entities (hashtag, url, or media)
	'''
	def contains_entity(self, entity_type, tweet):

	    if "entities" not in tweet:
	        return False
	    elif entity_type in tweet["entities"] and len(tweet["entities"][entity_type]) > 0:
	    	return True
	    return False

	'''
	    get an entity from a tweet if it exists
	'''
	def get_entity(self, entity_type, tweet):
	    if self.contains_entity(entity_type, tweet):
	        return [entity for entity in tweet["entities"][entity_type]]
	    return []

	'''
		gets a particular field for an entity if it exists
	'''
	def get_entity_field(self, field, entity):
		# beacuse all entities are actually lists
		# of entity objects
		for entity_object in entity:
			if field in entity_object:
				return entity[field]
		return None



import abc
import operator
from ..tools.tweet_parser import TweetParser

class BaseCollection(object):
	__metaclass__ = abc.ABCMeta

	@abc.abstractmethod
	def __init__(self):
		pass

	'''
		returns a list of test values for all 
		tweets, should return a dictionary
	'''
	def get_texts(self):
		return [tweet[text] for tweet in self]

	'''
		returns a dictionary with
		counts for the number of 
		top entities requested
	'''
	def top_entities(self, requested_entities):
		returndict = {}
		returnstructure = {}
		tweet_parser = TweetParser()
		#init dempty dict for all entity types
		for entity_type in requested_entities:
			returndict[entity_type] = {}

		for tweet in self.get_iterator():
			for entity_type in requested_entities:
				for entity in tweet_parser.get_entity(entity_type, tweet):
					if entity_type == 'user_mentions':
						entity_value = tweet_parser.get_entity_field('id_str', entity)
					elif entity_type == 'hashtags' or entity_type == 'symbols':
						entity_value = tweet_parser.get_entity_field('text', entity)
					else:
						entity_value = tweet_parser.get_entity_field('url', entity)

					if entity_value in returndict[entity_type]:
						returndict[entity_type][entity_value] += 1
					else:
						returndict[entity_type][entity_value] = 1

		for entity_type in returndict:
			if len(returndict[entity_type]) < 1:
				returnstructure[entity_type] = {}
			else:
				sorted_list = sorted(returndict[entity_type].items(), key=operator.itemgetter(1), reverse=True)
				# if the user put in 0 return all entites
				# otherwise slice the array and return the
				# number of top things they asked for
				# if the list is too short throw in None
				if requested_entities[entity_type] == 0:
					returnstructure[entity_type] = {name: count for name, count in sorted_list}
				elif len(sorted_list) < requested_entities[entity_type]:
					returnstructure[entity_type] = {name: count for name, count in sorted_list}
					for i in range(0,requested_entities[entity_type]-len(sorted_list)):
						returnstructure[entity_type][i] = None
				else:
					returnstructure[entity_type] = {name: count for name, count in sorted_list[0:requested_entities[entity_type]]}
		return returnstructure



import pymongo
from base_collection import BaseCollection

'''
replacement for: 
https://github.com/SMAPPNYU/smapp-toolkit/blob/master/smapp_toolkit/twitter/mongo_tweet_collection.py
'''

class MongoCollection(BaseCollection):
	'''
		method that tells us how to 
		create the MongoCollection object
	'''
	def __init__(self, address, port, username, password, database_name, collection_name):
		self.limit = 0
		self.mongo = pymongo.MongoClient(address, int(port))
		self.mongo_database = self.mongo[database_name]
		self.mongo_collection = self.mongo_database[collection_name]
		if username and password:
			self.mongo_database.authenticate(username, password)

	'''
		method that creates a cursor
		and yields all tweets in a particular collection
	'''
	def get_iterator(self):
		mongo_cursor = pymongo.cursor.Cursor(self.mongo_collection, self.filters(), no_cursor_timeout=True, limit=self.limit)
		try:
			for tweet in mongo_cursor:
				yield tweet
		finally:
			mongo_cursor.close()
	'''
	    Only return `count` tweets from the collection. Note: this takes tweets from the
	    beginning of the collection(s)
	    Example:
	    ########
	    collection.limit(5).texts()
	'''
	def limit(self, tweet_limit):
		self.limit = tweet_limit
		return self

	def filters(self):
		return {}
